@@ -91,6 +91,9 @@ metaprop <- function(event, n, studlab,
                   "Effect measure used:", sm))
   ##if (!is.logical(freeman.tukey))
   ##  stop("Parameter freeman.tukey must be of type 'logical'")
+  ##
+  if (any(n < 10) & sm=="PFT")
+    warning("Sample size very small (below 10) in at least one study. Accordingly, backtransformation for pooled effect may be misleading for Freeman-Tukey transformation. Please look at results for other transformations (e.g. sm='PAS' or sm='PLOGIT'), too.")
   
   
   ##
